# The LearningOnline Network with CAPA
# The LON-CAPA Homework handler
#
# $Id: lonhomework.pm,v 1.214 2005/07/14 02:40:10 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/


package Apache::lonhomework;
use strict;
use Apache::style();
use Apache::lonxml();
use Apache::lonnet;
use Apache::lonplot();
use Apache::inputtags();
use Apache::structuretags();
use Apache::randomlabel();
use Apache::response();
use Apache::hint();
use Apache::outputtags();
use Apache::caparesponse();
use Apache::radiobuttonresponse();
use Apache::optionresponse();
use Apache::imageresponse();
use Apache::essayresponse();
use Apache::externalresponse();
use Apache::rankresponse();
use Apache::matchresponse();
use Apache::chemresponse();
use Apache::drawimage();
use Apache::Constants qw(:common);
use HTML::Entities();
use Apache::loncommon();
use Apache::lonlocal;
use Time::HiRes qw( gettimeofday tv_interval );
use Apache::lonnet();

# FIXME - improve commenting


BEGIN {
    &Apache::lonxml::register_insert();
}


#
# Decides what targets to render for.
# Implicit inputs:
#   Various session environment variables:
#      request.state -  published  - is a /res/ resource
#                       uploaded   - is a /uploaded/ resource
#                       contruct   - is a /priv/ resource
#      form.grade_target - a form parameter requesting a specific target
sub get_target {
    &Apache::lonxml::debug("request.state = $env{'request.state'}");
    if( defined($env{'form.grade_target'})) {
	&Apache::lonxml::debug("form.grade_target= $env{'form.grade_target'}");
    } else {
	&Apache::lonxml::debug("form.grade_target <undefined>");
    }
    if (($env{'request.state'} eq "published") ||
	($env{'request.state'} eq "uploaded")) {
	if ( defined($env{'form.grade_target'}  ) 
	     && ($env{'form.grade_target'} eq 'tex')) {
	    return ($env{'form.grade_target'});
	} elsif ( defined($env{'form.grade_target'}  ) 
		  && ($Apache::lonhomework::viewgrades eq 'F' )) {
	    return ($env{'form.grade_target'});
	}
	if ($env{'form.webgrade'} &&
	    $Apache::lonhomework::modifygrades eq 'F') {
	    return ('grade','webgrade');
	}
	if ( defined($env{'form.submitted'}) &&
	     ( !defined($env{'form.resetdata'})) &&
	     ( !defined($env{'form.newrandomization'}))) {
	    return ('grade', 'web');
	} else {
	    return ('web');
	}
    } elsif ($env{'request.state'} eq "construct") {
	if ( defined($env{'form.grade_target'}) ) {
	    return ($env{'form.grade_target'});
	}
	if ( defined($env{'form.preview'})) {
	    if ( defined($env{'form.submitted'})) {
		return ('grade', 'web');
	    } else {
		return ('web');
	    }
	} else {
	    if ( $env{'form.problemmode'} eq &mt('View') ||
		 $env{'form.problemmode'} eq &mt('Discard Edits and View')) {
		if ( defined($env{'form.submitted'}) &&
		     (!defined($env{'form.resetdata'})) &&
		     (!defined($env{'form.newrandomization'}))) {
		    return ('grade', 'web','answer');
		} else {
		    return ('web','answer');
		}
	    } elsif ( $env{'form.problemmode'} eq &mt('Edit') ||
		      $env{'form.problemmode'} eq 'Edit') {
		if ( $env{'form.submitted'} eq 'edit' ) {
		    if ( $env{'form.submit'} eq &mt('Submit Changes and View') ) {
			return ('modified','web','answer');
		    } else {
			return ('modified','no_output_web','edit');
		    }
		} else {
		    return ('no_output_web','edit');
		}
	    } else {
		return ('web');
	    }
	}
    }
    return ();
}

sub setup_vars {
    my ($target) = @_;
    return ';'
#  return ';$external::target='.$target.';';
}

sub send_header {
    my ($request)= @_;
    $request->print(&Apache::lontexconvert::header());
#  $request->print('<form name='.$env{'form.request.prefix'}.'lonhomework method="POST" action="'.$request->uri.'">');
}

sub createmenu {
    my ($which,$request)=@_;
    if ($which eq 'grade') {
	$request->print('<script language="JavaScript"> 
          hwkmenu=window.open("/res/adm/pages/homeworkmenu.html","homeworkremote",
                 "height=350,width=150,menubar=no");
          </script>');
    }
}

sub send_footer {
    my ($request)= @_;
#  $request->print('</form>');
    $request->print(&Apache::lontexconvert::footer());
}

sub proctor_checked_in {
    my ($slot_name,$slot)=@_;
    my @allowed=split(",",$slot->{'proctor'});
    my $version=$Apache::lonhomework::history{'resource.version'};
    foreach my $possible (@allowed) { 
	if ($Apache::lonhomework::history{"resource.$version.checkedin"} eq
	    $possible &&
	    $Apache::lonhomework::history{"resource.$version.checkedin.slot"}
	       eq $slot_name) {
	    return 1;
	}
    }
    return 0;
}

$Apache::lonxml::browse='';
sub check_ip_acc {
    my ($acc)=@_;
    if (!defined($acc) || $acc =~ /^\s*$/) { return 1; }
    my $allowed=0;
    my $ip=$ENV{'REMOTE_ADDR'};
    my $name;
    foreach my $pattern (split(',',$acc)) {
	if ($pattern =~ /\*$/) {
	    #35.8.*
	    $pattern=~s/\*//;
	    if ($ip =~ /^\Q$pattern\E/) { $allowed=1; }
	} elsif ($pattern =~ /(\d+\.\d+\.\d+)\.\[(\d+)-(\d+)\]$/) {    
	    #35.8.3.[34-56]
	    my $low=$2;
	    my $high=$3;
	    $pattern=$1;
	    if ($ip =~ /^\Q$pattern\E/) { 
		my $last=(split(/\./,$ip))[3];
		if ($last <=$high && $last >=$low) { $allowed=1; }
	    }
	} elsif ($pattern =~ /^\*/) {
	    #*.msu.edu
	    $pattern=~s/\*//;
	    if (!defined($name)) {
		use Socket;
		my $netaddr=inet_aton($ip);
		($name)=gethostbyaddr($netaddr,AF_INET);
	    }
	    if ($name =~ /\Q$pattern\E$/i) { $allowed=1; }
	} elsif ($pattern =~ /\d+\.\d+\.\d+\.\d+/) {
	    #127.0.0.1
	    if ($ip =~ /^\Q$pattern\E/) { $allowed=1; }
	} else {
	    #some.name.com
	    if (!defined($name)) {
		use Socket;
		my $netaddr=inet_aton($ip);
		($name)=gethostbyaddr($netaddr,AF_INET);
	    }
	    if ($name =~ /\Q$pattern\E$/i) { $allowed=1; }
	}
	if ($allowed) { last; }
    }
    return $allowed;
}

sub check_task_access {
    # does it pass normal muster
    # yes we really do want the default args passing
    my ($status,$datemsg)=&check_access;
    if ($status eq 'SHOW_ANSWER' ||
	$status eq 'CLOSED' ||
	$status eq 'CANNOT_ANSWER' ||
	$status eq 'INVALID_ACCESS' ||
	$status eq 'UNAVAILABLE') {
	return ($status,$datemsg);
    }
    if ($env{'request.state'} eq "construct") {
	return ($status,$datemsg);
    }
    my $version=$Apache::lonhomework::history{'resource.version'};
    if ($Apache::lonhomework::history{"resource.$version.checkedin"} &&
	$Apache::lonhomework::history{"resource.$version.status"} eq 'pass') {
	return ('SHOW_ANSWER');
    }
    my ($id)=@_;
    my @slots=
	(split(':',&Apache::lonnet::EXT("resource.$id.availablestudent")),
	 split(':',&Apache::lonnet::EXT("resource.$id.available")));

#    if (!@slots) {
#	return ($status,$datemsg);
#    }
    my $slotstatus='NOT_IN_A_SLOT';
    my ($returned_slot,$slot_name);
    foreach my $slot (@slots) {
	&Apache::lonxml::debug("getting $slot");
	my %slot=&Apache::lonnet::get_slot($slot);
	&Apache::lonhomework::showhash(%slot);
	if ($slot{'starttime'} < time &&
	    $slot{'endtime'} > time &&
	    &check_ip_acc($slot{'ip'})) {
	    &Apache::lonxml::debug("$slot is good");
	    $slotstatus='NEEDS_CHECKIN';
	    $returned_slot=\%slot;
	    $slot_name=$slot;
	    last;
	}
    }
    if ($slotstatus eq 'NEEDS_CHECKIN' &&
	&proctor_checked_in($slot_name,$returned_slot)) {
	&Apache::lonxml::debug("protoctor checked in");
	$slotstatus='CAN_ANSWER';
    }
    if ( $slotstatus eq 'NOT_IN_A_SLOT' && 
	$Apache::lonhomework::history{"resource.$version.checkedin"}) {
	if ($Apache::lonhomework::history{"resource.$version.status"} eq 'fail') {
	    return ('SHOW_ANSWER');
	} else {
	    return ('WAITING_FOR_GRADE');
	}
    }
    return ($slotstatus,$datemsg,$slot_name,$returned_slot);
}

# JB, 9/24/2002: Any changes in this function may require a change
# in lonnavmaps::resource::getDateStatus.
sub check_access {
    my ($id) = @_;
    my $date ='';
    my $status;
    my $datemsg = '';
    my $lastdate = '';
    my $temp;
    my $type;
    my $passed;

    if ($env{'request.state'} eq "construct") {
	if ($env{'form.problemstate'}) {
	    if ($env{'form.problemstate'} =~ /^CANNOT_ANSWER/) {
		if ( ! ($env{'form.problemstate'} eq 'CANNOT_ANSWER_correct' &&
			lc($Apache::lonhomework::problemstatus) eq 'no')) {
		    return ('CANNOT_ANSWER',
			    &mt('is in this state due to author settings.'));
		}
	    } else {
		return ($env{'form.problemstate'},
			&mt('is in this state due to author settings.'));
	    }
	}
	&Apache::lonxml::debug("in construction ignoring dates");
	$status='CAN_ANSWER';
	$datemsg=&mt('is in under construction');
#	return ($status,$datemsg);
    }

    &Apache::lonxml::debug("checking for part :$id:");
    &Apache::lonxml::debug("time:".time);

    my ($symb)=&Apache::lonxml::whichuser();
    &Apache::lonxml::debug("symb:".$symb);
    #if ($env{'request.state'} ne "construct" && $symb ne '') {
    if ($env{'request.state'} ne "construct") {
	my $allowed=&check_ip_acc(&Apache::lonnet::EXT("resource.$id.acc"));
	if (!$allowed && ($Apache::lonhomework::browse ne 'F')) {
	    $status='INVALID_ACCESS';
	    $date=&mt("can not be accessed from your location.");
	    return($status,$date);
	}
	
	foreach $temp ("opendate","duedate","answerdate") {
	    $lastdate = $date;
	    $date = &Apache::lonnet::EXT("resource.$id.$temp");
	    my $thistype = &Apache::lonnet::EXT("resource.$id.$temp.type");
	    if ($thistype =~ /^(con_lost|no_such_host)/ ||
		$date     =~ /^(con_lost|no_such_host)/) {
		$status='UNAVAILABLE';
		$date=&mt("may open later.");
		return($status,$date);
	    }
	    if ($thistype eq 'date_interval') {
		if ($temp eq 'opendate') {
		    $date=&Apache::lonnet::EXT("resource.$id.duedate")-$date;
		}
		if ($temp eq 'answerdate') {
		    $date=&Apache::lonnet::EXT("resource.$id.duedate")+$date;
		}
	    }
	    &Apache::lonxml::debug("found :$date: for :$temp:");
	    if ($date eq '') {
		$date = &mt("an unknown date"); $passed = 0;
	    } elsif ($date eq 'con_lost') {
		$date = &mt("an indeterminate date"); $passed = 0;
	    } else {
		if (time < $date) { $passed = 0; } else { $passed = 1; }
		$date = localtime $date;
	    }
	    if (!$passed) { $type=$temp; last; }
	}
	&Apache::lonxml::debug("have :$type:$passed:");
	if ($passed) {
	    $status='SHOW_ANSWER';
	    $datemsg=$date;
	} elsif ($type eq 'opendate') {
	    $status='CLOSED';
	    $datemsg = &mt("will open on")." $date";
	} elsif ($type eq 'duedate') {
	    $status='CAN_ANSWER';
	    $datemsg = &mt("is due at")." $date";
	} elsif ($type eq 'answerdate') {
	    $status='CLOSED';
	    $datemsg = &mt("was due on")." $lastdate".&mt(", and answers will be available on")." $date";
	}
    }
    if ($status eq 'CAN_ANSWER' ||
	(($Apache::lonhomework::browse eq 'F') && ($status eq 'CLOSED'))) {
	#check #tries, and if correct.
	my $tries = $Apache::lonhomework::history{"resource.$id.tries"};
	my $maxtries = &Apache::lonnet::EXT("resource.$id.maxtries");
	if ( $tries eq '' ) { $tries = '0'; }
	if ( $maxtries eq '' && 
	     $env{'request.state'} ne 'construct') { $maxtries = '2'; } 
	if ($maxtries && $tries >= $maxtries) { $status = 'CANNOT_ANSWER'; }
	# if (correct and show prob status) or excused then CANNOT_ANSWER
	if(($Apache::lonhomework::history{"resource.$id.solved"}=~/^correct/
	    &&
	    lc($Apache::lonhomework::problemstatus) ne 'no')
	   ||
	   $Apache::lonhomework::history{"resource.$id.solved"}=~/^excused/) {
	    $status = 'CANNOT_ANSWER';
	}
    }
    if ($status eq 'CAN_ANSWER' || $status eq 'CANNOT_ANSWER') {
	my $interval=&Apache::lonnet::EXT("resource.$id.interval");
	&Apache::lonxml::debug("looking for interval $interval");
	if ($interval) {
	    my $first_access=&Apache::lonnet::get_first_access('map');
	    &Apache::lonxml::debug("looking for accesstime $first_access");
	    if (!$first_access) {
		$status='NOT_YET_VIEWED';
		$datemsg=&seconds_to_human_length($interval);
	    } else {
		my $newdate=localtime($first_access+$interval);
		if (time > ($first_access+$interval)) {
		    $status='CLOSED';
		    $datemsg = &mt("was due on")." $newdate".&mt(", and answers will be available on")." $date";
		} else {
		    $datemsg = &mt("is due at")." $newdate";
		}
	    }
	}
    }
  #if (($status ne 'CLOSED') && ($Apache::lonhomework::type eq 'exam') &&
  #    (!$Apache::lonhomework::history{"resource.0.outtoken"})) {
  #    return ('UNCHECKEDOUT','needs to be checked out');
  #}


    &Apache::lonxml::debug("sending back :$status:$datemsg:");
    if (($Apache::lonhomework::browse eq 'F') && ($status eq 'CLOSED')) {
	&Apache::lonxml::debug("should be allowed to browse a resource when closed");
	$status='CAN_ANSWER';
	$datemsg=&mt('is closed but you are allowed to view it');
    }

    return ($status,$datemsg);
}

sub seconds_to_human_length {
    my ($length)=@_;

    my $seconds=$length%60; $length=int($length/60);
    my $minutes=$length%60; $length=int($length/60);
    my $hours=$length%24;   $length=int($length/24);
    my $days=$length;

    my $timestr;
    if ($days > 0) { $timestr.=&mt('[quant,_1,day]',$days); }
    if ($hours > 0) { $timestr.=($timestr?", ":"").
			  &mt('[quant,_1,hour]',$hours); }
    if ($minutes > 0) { $timestr.=($timestr?", ":"").
			    &mt('[quant,_1,minute]',$minutes); }
    if ($seconds > 0) { $timestr.=($timestr?", ":"").
			    &mt('[quant,_1,second]',$seconds); }
    return $timestr;
}

sub showhash {
    my (%hash) = @_;
    &showhashsubset(\%hash,'.');
    return '';
}

sub showarray {
    my ($array)=@_;
    my $string="(";
    foreach my $elm (@{ $array }) {
	if (ref($elm) eq 'ARRAY') {
	    $string.=&showarray($elm);
	} elsif (ref($elm) eq 'HASH') {
	    $string.= "HASH --- \n<br />";
	    $string.= &showhashsubset($elm,'.');
	} else {
	    $string.="$elm,"
	}
    }
    chop($string);
    $string.=")";
    return $string;
}

sub showhashsubset {
    my ($hash,$keyre) = @_;
    my $resultkey;
    foreach $resultkey (sort keys %$hash) {
	if ($resultkey !~ /$keyre/) { next; }
	if (ref($$hash{$resultkey})  eq 'ARRAY' ) {
	    &Apache::lonxml::debug("$resultkey ---- ".
				   &showarray($$hash{$resultkey}));
	} elsif (ref($$hash{$resultkey}) eq 'HASH' ) {
	    &Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
	    &showhashsubset($$hash{$resultkey},'.');
	} else {
	    &Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
	}
    }
    &Apache::lonxml::debug("\n<br />restored values^</br>\n");
    return '';
}

sub setuppermissions {
    $Apache::lonhomework::browse= &Apache::lonnet::allowed('bre',$env{'request.filename'});
    my $viewgrades = &Apache::lonnet::allowed('vgr',$env{'request.course.id'});
    if (! $viewgrades && 
	exists($env{'request.course.sec'}) && 
	$env{'request.course.sec'} !~ /^\s*$/) {
	$viewgrades = &Apache::lonnet::allowed('vgr',$env{'request.course.id'}.
                                               '/'.$env{'request.course.sec'});
    }
    $Apache::lonhomework::viewgrades = $viewgrades; # File global variable...dirt.
    if ($Apache::lonhomework::browse eq 'F' && 
	$env{'form.devalidatecourseresdata'} eq 'on') {
	my (undef,$courseid) = &Apache::lonxml::whichuser();
	&Apache::lonnet::devalidatecourseresdata($env{"course.$courseid.num"},
					      $env{"course.$courseid.domain"});
    }
    my $modifygrades = &Apache::lonnet::allowed('mgr',$env{'request.course.id'});
    if (! $modifygrades && 
	exists($env{'request.course.sec'}) && 
	$env{'request.course.sec'} !~ /^\s*$/) {
	$modifygrades = 
	    &Apache::lonnet::allowed('mgr',$env{'request.course.id'}.
				     '/'.$env{'request.course.sec'});
    }
    $Apache::lonhomework::modifygrades = $modifygrades;
    return '';
}

sub setupheader {
    my $request=$_[0];
    &Apache::loncommon::content_type($request,'text/html');
    if (!$Apache::lonxml::debug && ($ENV{'REQUEST_METHOD'} eq 'GET')) {
	&Apache::loncommon::no_cache($request);
    }
#    $request->set_last_modified(&Apache::lonnet::metadata($request->uri,
#							  'lastrevisiondate'));
    $request->send_http_header;
    return OK if $request->header_only;
    return ''
}

sub handle_save_or_undo {
    my ($request,$problem,$result) = @_;
    my $file    = &Apache::lonnet::filelocation("",$request->uri);
    my $filebak =$file.".bak";
    my $filetmp =$file.".tmp";
    my $error=0;

    &Apache::lonnet::correct_line_ends($result);

    if ($env{'form.Undo'} eq &mt('undo')) {
	my $error=0;
	if (!copy($file,$filetmp)) { $error=1; }
	if ((!$error) && (!copy($filebak,$file))) { $error=1; }
	if ((!$error) && (!move($filetmp,$filebak))) { $error=1; }
	if (!$error) {
	    &Apache::lonxml::info("<p><b>".&mt("Undid changes, Switched")." $filebak ".&mt("and")." $file</b></p>");
	} else {
	    &Apache::lonxml::info("<p><font color=\"red\" size=\"+1\"><b>".&mt("Unable to undo, unable to switch")." $filebak ".&mt("and")." $file</b></font></p>");
	    $error=1;
	}
    } else {
	my $fs=Apache::File->new(">$filebak");
	if (defined($fs)) {
	    print $fs $$problem;
	    &Apache::lonxml::info("<b>".&mt("Making Backup to").
				  " $filebak</b>");
	} else {
	    &Apache::lonxml::info("<font color=\"red\" size=\"+1\"><b>".&mt("Unable to make backup")." $filebak</b></font>");
	    $error=2;
	}
	my $fh=Apache::File->new(">$file");
	if (defined($fh)) {
	    print $fh $$result;
	    &Apache::lonxml::info("<b>".&mt("Saving Modifications to").
				  " $file</b>");
	} else {
	    &Apache::lonxml::info("<font color=\"red\" size=\"+1\"><b>".
				  &mt("Unable to write to")." $file</b></font>");
	    $error|=4;
	}
    }
    return $error;
}

sub analyze_header {
    my ($request) = @_;
    my $bodytag='<body bgcolor="#ffffff">';
    if ($env{'environment.remote'} eq 'off') {
	$bodytag=&Apache::loncommon::bodytag();
    }
    my $html=&Apache::lonxml::xmlbegin();
    my $result.=$html.'
            <head><title>'.&mt("Analyzing a problem").'</title></head>
            '.$bodytag.&Apache::lonxml::message_location().'
            <form name="lonhomework" method="POST" action="'.
	    &HTML::Entities::encode($env{'request.uri'},'<>&"').'">'.
	    &Apache::structuretags::remember_problem_state().'
            <input type="submit" name="problemmode" value="'.&mt("EditXML").'" />
            <input type="submit" name="problemmode" value="'.&mt('Edit').'" />
            <hr />
            <input type="submit" name="submit" value="'.&mt("View").'" />
            <hr />
            </form>';
    &Apache::lonxml::add_messages(\$result);
    $request->print($result);
    $request->rflush();
}

sub analyze_footer {
    my ($request) = @_;
    my $result='</body></html>';
    $request->print($result);
    $request->rflush();
}

sub analyze {
    my ($request,$file) = @_;
    &Apache::lonxml::debug("Analyze");
    my $result;
    my %overall;
    my %allparts;
    my $rndseed=$env{'form.rndseed'};
    &analyze_header($request);
    my %prog_state=
	&Apache::lonhtmlcommon::Create_PrgWin($request,&mt('Analyze Progress'),
					      &mt('Getting Problem Variants'),
					      $env{'form.numtoanalyze'},
					      'inline',undef);
    for(my $i=1;$i<$env{'form.numtoanalyze'}+1;$i++) {
	&Apache::lonhtmlcommon::Increment_PrgWin($request,\%prog_state,
						 &mt('last problem'));
	if (&Apache::loncommon::connection_aborted($request)) { return; }
	my $subresult=&Apache::lonnet::ssi($request->uri,
					   ('grade_target' => 'analyze'),
					   ('rndseed' => $i+$rndseed));
	(my $garbage,$subresult)=split(/_HASH_REF__/,$subresult,2);
	my %analyze=&Apache::lonnet::str2hash($subresult);
	my @parts;
	if (defined(@{ $analyze{'parts'} })) {
	    @parts=@{ $analyze{'parts'} };
	}
	foreach my $part (@parts) {
	    if (!exists($allparts{$part})) {$allparts{$part}=1;};
	    if ($analyze{$part.'.type'} eq 'numericalresponse'	||
		$analyze{$part.'.type'} eq 'stringresponse'	||
		$analyze{$part.'.type'} eq 'formularesponse'   ) {
		push( @{ $overall{$part.'.answer'} },
		      [@{ $analyze{$part.'.answer'} }]);
	    }
	}
    }
    &Apache::lonhtmlcommon::Update_PrgWin($request,\%prog_state,
					  &mt('Analyzing Results'));
    $request->print('<hr />'.&mt('List of possible answers').': ');
    foreach my $part (sort(keys(%allparts))) {
	if (defined(@{ $overall{$part.'.answer'} })) {
	    my $num_cols=scalar(@{ $overall{$part.'.answer'}->[0] });
	    $request->print('<table><tr><th colspan="'.($num_cols+1).'">'.&mt('Part').' '.$part.'</th></tr>');
	    my %frequency;
	    foreach my $answer (sort {$a->[0] <=> $b->[0]} (@{ $overall{$part.'.answer'} })) {
		$frequency{join("\0",@{ $answer })}++;
	    }
	    $request->print('<tr><th colspan="'.($num_cols).'">'.&mt('Answer').'</th><th>'.&mt('Frequency').'</th></tr>');
	    foreach my $answer (sort {(split("\0",$a))[0] <=> (split("\0",$b))[0]} (keys(%frequency))) {
		$request->print('<tr><td align="right">'.
				join('</td><td align="right">',split("\0",$answer)).
				'</td><td>('.$frequency{$answer}.
				')</td></tr>');
	    }
	    $request->print('</table>');
	} else {
	    $request->print('<p>'.&mt('Response').' '.$part.' '.
			    &mt('is not analyzable at this time').'</p>');
	}
    }
    if (scalar(keys(%allparts)) == 0 ) {
	$request->print('<p>'.&mt('Found no analyzable respones in this problem, currently only Numerical, Formula and String response styles are supported.').'</p>');
    }
    &Apache::lonhtmlcommon::Close_PrgWin($request,\%prog_state);
    &analyze_footer($request);
    &Apache::lonhomework::showhash(%overall);
    return $result;
}

sub editxmlmode {
    my ($request,$file) = @_;
    my $result;
    my $problem=&Apache::lonnet::getfile($file);
    if ($problem eq -1) {
	&Apache::lonxml::error("<b> ".&mt('Unable to find').
			       " <i>$file</i></b>");
	$problem='';
    }
    if (defined($env{'form.editxmltext'}) || defined($env{'form.Undo'})) {
	my $error=&handle_save_or_undo($request,\$problem,
				       \$env{'form.editxmltext'});
	if (!$error) { $problem=&Apache::lonnet::getfile($file); }
    }
    &Apache::lonhomework::showhashsubset(\%env,'^form');
    if ( $env{'form.submit'} eq &mt('Submit Changes and View') ) {
	&Apache::lonhomework::showhashsubset(\%env,'^form');
	$env{'form.problemmode'}='View';
	&renderpage($request,$file);
    } else {
	my ($rows,$cols) = &Apache::edit::textarea_sizes(\$problem);
	my $xml_help = '<table><tr><td>'.
	    &Apache::loncommon::helpLatexCheatsheet("Problem_Editor_XML_Index",
						    "Problem Editing Help").
						    '</td><td>'.
       &Apache::loncommon::help_open_menu('',undef,undef,undef,5,'Authoring').
                '</td></tr></table>';
	if ($cols > 80) { $cols = 80; }
	if ($cols < 70) { $cols = 70; }
	if ($rows < 20) { $rows = 20; }
	my $bodytag='<body bgcolor="#ffffff">';
	if ($env{'environment.remote'} eq 'off') {
	    $bodytag=&Apache::loncommon::bodytag();
	}
	my $html=&Apache::lonxml::xmlbegin();
	$result.=$html.$bodytag.
	    &renderpage($request,$file,['no_output_web'],1).
	    &Apache::lonxml::message_location().'
            <form name="lonhomework" method="POST" action="'.
	    &HTML::Entities::encode($env{'request.uri'},'<>&"').'">'.
	    &Apache::structuretags::remember_problem_state().'
            <input type="hidden" name="problemmode" value="'.&mt('EditXML').'" />
            <input type="submit" name="problemmode" accesskey="d" value="'.&mt('Discard Edits and View').'" />
            <input type="submit" name="problemmode" accesskey="e" value="'.&mt('Edit').'" />
            <hr />
            <input type="submit" name="submit" accesskey="s" value="'.&mt('Submit Changes').'" />
            <input type="submit" name="submit" accesskey="v" value="'.&mt('Submit Changes and View').'" />
            <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
            <hr />
            ' . $xml_help . '
            <textarea style="width:100%" rows="'.$rows.'" cols="'.$cols.'" name="editxmltext">'.
	    &HTML::Entities::encode($problem,'<>&"').'</textarea>
            </form></body></html>';
	&Apache::lonxml::add_messages(\$result);
	$request->print($result);
    }
    return '';
}

#
#    Render the page in whatever target desired.
#
sub renderpage {
    my ($request,$file,$targets,$return_string) = @_;

    my @targets = @{$targets || [&get_target()]};
    &Apache::lonhomework::showhashsubset(\%env,'form.');
    &Apache::lonxml::debug("Running targets ".join(':',@targets));
    my $overall_result;
    foreach my $target (@targets) {
	# FIXME need to do something intelligent when a problem goes
        # from viewable to not viewable due to map conditions
	#&setuppermissions();
	#if (   $Apache::lonhomework::browse ne '2'
	#    && $Apache::lonhomework::browse ne 'F' ) {
	#    $request->print(" You most likely shouldn't see me.");
	#}
	#my $t0 = [&gettimeofday()];
	my $output=1;
	if ($target eq 'no_output_web') {
	    $target = 'web'; $output=0;
	}
	my $problem=&Apache::lonnet::getfile($file);
	if ($problem eq -1) {
	    &Apache::lonxml::error("<b> ".&mt('Unable to find')." <i>$file</i></b>");
	    $problem='';
	}

	my %mystyle;
	if ($target eq 'analyze') { %Apache::lonhomework::analyze=(); }
	if ($target eq 'answer') { &showhash(%Apache::lonhomework::history); }
	if ($target eq 'web') {&Apache::lonhomework::showhashsubset(\%env,'^form');}

	&Apache::lonxml::debug("Should be parsing now");
	my $result = &Apache::lonxml::xmlparse($request, $target, $problem,
					       &setup_vars($target),%mystyle);
	undef($Apache::lonhomework::parsing_a_problem);
	if (!$output) { $result = ''; }
	#$request->print("Result follows:");
	if ($target eq 'modified') {
	    &handle_save_or_undo($request,\$problem,\$result);
	} else {
	    if ($target eq 'analyze') {
		$result=&Apache::lonnet::hashref2str(\%Apache::lonhomework::analyze);
		undef(%Apache::lonhomework::analyze);
	    }
	    #my $td=&tv_interval($t0);
	    #if ( $Apache::lonxml::debug) {
	    #$result =~ s:</body>::;
	    #$result.="<br />Spent $td seconds processing target $target\n</body>";
	    #}
#	    $request->print($result);
	    $overall_result.=$result;
#	    $request->rflush();
	}
	#$request->print(":Result ends");
	#my $td=&tv_interval($t0);
    }
    if (!$return_string) {
	&Apache::lonxml::add_messages(\$overall_result);
	$request->print($overall_result);   
	$request->rflush();   
    } else {
	return $overall_result;
    }
}

# with no arg it returns a HTML <option> list of the template titles
# with one arg it returns the filename associated with the arg passed
sub get_template_list {
    my ($namewanted,$extension) = @_;
    my $result;
    my @allnames;
    &Apache::lonxml::debug("Looking for :$extension:");
    foreach my $file (</home/httpd/html/res/adm/includes/templates/*.$extension>) {
	my $name=&Apache::lonnet::metadata($file,'title');
	if ($namewanted && ($name eq $namewanted)) {
	    $result=$file;
	    last;
	} else {
	    if ($name) { push (@allnames, $name); }
	}
    }
    if (@allnames && !$result) {
	$result="<option>".&mt("Select a")." $extension ".&mt('template')."</option>\n<option>".
	    join('</option><option>',sort(@allnames)).'</option>';
    }
    return $result;
}

sub newproblem {
    my ($request) = @_;
    my $extension=$request->uri;
    $extension=~s:^.*\.([\w]+)$:$1:;
    &Apache::lonxml::debug("Looking for :$extension:");
    my $templatelist=&get_template_list('',$extension);
    if ($env{'form.template'} &&
	$env{'form.template'} ne "Select a $extension template") {
	use File::Copy;
	my $file = &get_template_list($env{'form.template'},$extension);
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	copy($file,$dest);
	&renderpage($request,$dest);
    } elsif($env{'form.newfile'} && !$templatelist) {
	# I don't like hard-coded filenames but for now, this will work.
	use File::Copy;
	my $templatefilename =
	    $request->dir_config('lonIncludes').'/templates/blank.problem';
	&Apache::lonxml::debug("$templatefilename");
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	copy($templatefilename,$dest);
	&renderpage($request,$dest);
    } else {
	my $url=&HTML::Entities::encode($request->uri,'<>&"');
	my $shownurl=$url;	
	$shownurl=~s-^/~-/priv/-;
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	my $errormsg;
	if ($env{'form.newfile'}) {
	    $errormsg='<p><font color="red">'.&mt('You did not select a template.').'</font></p>'."\n";
	}
	my $instructions;
	my $bodytag=&Apache::loncommon::bodytag(undef,undef,undef,
					($env{'environment.remote'} ne 'off'));
	if ($templatelist) { $instructions=&mt(", select a template from the pull-down menu below.").'<br />'.&mt("Then");}
	my %lt=&Apache::lonlocal::texthash( 'create' => 'Creating a new',
			  'resource' => 'resource',
			  'requested' => 'The requested file',
			  'not exist' => 'currently does not exist',
			  'createnew' => 'To create a new',
			  'click' => 'click on the',
			  'Create' => 'Create',
			  'button' => 'button');
	$request->print(<<ENDNEWPROBLEM);
$bodytag
<h1>$lt{'create'} $extension $lt{'resource'}</h1>
$errormsg
$lt{'requested'} <tt>$shownurl</tt> $lt{'not exist'}.
<p>
<b>$lt{'createnew'} $extension$instructions $lt{'click'} "$lt{'Create'} $extension" $lt{'button'}.</b>
</p>
<p><form action="$url" method="POST">
ENDNEWPROBLEM
	if (defined($templatelist)) {
	    $request->print("<select name=\"template\">$templatelist</select>");
	}
	$request->print("<br /><input type=\"submit\" name=\"newfile\" value=\"".&mt('Create')." $extension\" />");
	$request->print("</form></p></body>");
    }
    return '';
}

sub view_or_edit_menu {
    my ($request) = @_;
    my $url=&HTML::Entities::encode($request->uri,'<>&"');
    my %lt=&Apache::lonlocal::texthash( 'would' => 'Would you like to',
		      'view' => 'View',
		      'Edit' => 'edit',
		      'or' => 'or',
		      'the problem' => 'the problem');
    $request->print(<<EDITMENU);
<body bgcolor="#FFFFFF">
<form action="$url" method="POST">
$lt{'would'} <input type="submit" name="problemmode" accesskey="v" value="&lt{'view'}">
&lt{'or'} <input type="submit" name="problemmode" accesskey="e" value="&lt{'Edit'}">
&lt{'the problem'}.
</form>
</body>
EDITMENU
}

sub handler {
    #my $t0 = [&gettimeofday()];
    my $request=$_[0];
    
    $Apache::lonxml::debug=$env{'user.debug'};
    $env{'request.uri'}=$request->uri;
    &setuppermissions();
    # some times multiple problemmodes are submitted, need to select
    # the last one
    if ( defined($env{'form.problemmode'}) && ref($env{'form.problemmode'}) ) {
	my $mode=$env{'form.problemmode'}->[-1];
	undef $env{'form.problemmode'};
	$env{'form.problemmode'}=$mode;
    }

    my $file=&Apache::lonnet::filelocation("",$request->uri);

    #check if we know where we are
    if ($env{'request.course.fn'} && !&Apache::lonnet::symbread()) { 
	# if we are browsing we might not be able to know where we are
	if ($Apache::lonhomework::browse ne 'F' && 
	    $env{'request.state'} ne "construct") {
	    #should know where we are, so ask
	    if ( &Apache::lonnet::mod_perl_version() == 2 ) {
		&Apache::lonnet::cleanenv();
	    }
	    &Apache::lonnet::logthis(&Apache::lonnet::mod_perl_version());
	    $request->internal_redirect('/adm/ambiguous'); return OK;
	}
    }
    if (&setupheader($request)) { return OK; }
    &Apache::lonxml::debug("Permissions:$Apache::lonhomework::browse:$Apache::lonhomework::viewgrades:");
    &Apache::lonxml::debug("Problem Mode ".$env{'form.problemmode'});
    my ($symb) = &Apache::lonxml::whichuser();
    &Apache::lonxml::debug('symb is '.$symb);
    if ($env{'request.state'} eq "construct" || $symb eq '') {
	if ($env{'form.resetdata'} eq &mt('Reset Submissions') ||
	    $env{'form.resetdata'} eq &mt('New Problem Variation') ||
	    $env{'form.newrandomization'} eq &mt('New Randomization')) {
	    my ($symb,$courseid,$domain,$name) = &Apache::lonxml::whichuser();
	    &Apache::lonnet::tmpreset($symb,'',$domain,$name);
	    &Apache::lonxml::debug("Attempt reset");
	}
    }
    if ($env{'request.state'} eq "construct") {
	if ( -e $file ) {
	    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
						    ['problemmode']);
	    if (!(defined $env{'form.problemmode'})) {
		#first visit to problem in construction space
		#&view_or_edit_menu($request);
		$env{'form.problemmode'}='View';
		&renderpage($request,$file);
	    } elsif ($env{'form.problemmode'} eq &mt('EditXML') ||
		     $env{'form.problemmode'} eq 'EditXML') {
		&editxmlmode($request,$file);
	    } elsif ($env{'form.problemmode'} eq &mt('Calculate answers')) {
		&analyze($request,$file);
	    } else {
		&renderpage($request,$file);
	    }
	} else {
	    # requested file doesn't exist in contruction space
	    &newproblem($request);
	}
    } else {
	# just render the page normally outside of construction space
	&Apache::lonxml::debug("not construct");
	&renderpage($request,$file);
    }
    #my $td=&tv_interval($t0);
    #&Apache::lonxml::debug("Spent $td seconds processing");
    # &Apache::lonhomework::send_footer($request);
    # always turn off debug messages
    $Apache::lonxml::debug=0;
    return OK;

}

1;
__END__
