# The LearningOnline Network with CAPA
# generate frame-based help system
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonhelpmenu;

use strict;
use lib qw(/home/httpd/lib/perl);
use Apache::Constants qw(:common);
use Apache::loncommon();
use Apache::lonlocal;
use Apache::lonnet;

sub handler {
    my ($r) = @_;
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['page','color','function','faq','bug','topic','component_help','origurl','stayonpage']);
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;

    if ($r->header_only) {
        return OK;
    }
    my $color = $env{'form.color'};
    my $faq = $env{'form.faq'};
    my $bug = $env{'form.bug'};
    my $topic = $env{'form.topic'};
    my $function = $env{'form.function'};
    my $component_help = $env{'form.component_help'};
    my $origurl = $env{'form.origurl'};
    my $stayOnPage = $env{'form.stayonpage'};
    my $component_url = $component_help;
    if ($component_url) {
        $component_url = '/adm/help/'.$component_url.'.hlp';
    }
    my $bugurl = $Apache::lonnet::perlvar{'BugzillaHost'};
    $bugurl .= 'enter_bug.cgi?product=LON-CAPA&bug_file_loc='.$origurl;
    if ($bug) {
        $bugurl .= '&component='.$bug;
    }
    my $faqbaseurl = $Apache::lonnet::perlvar{'FAQHost'};
    my $requestmail = $Apache::lonnet::perlvar{'lonSupportEMail'};
    if ($env{'form.page'} eq 'banner') {
        &display_help_banner($r,$color,$function,$faq,$bug,$topic,$component_url,$origurl,$bugurl,$faqbaseurl,$requestmail,$stayOnPage);
    } elsif ($env{'form.page'} eq 'body') {
        &display_help_mainpage($r,$color,$function,$faq,$bug,$topic,$component_url,$origurl,$bugurl,$faqbaseurl,$requestmail);
    }
    return OK;
}

sub display_help_banner {
    my ($r,$color,$function,$faq,$bug,$topic,$component_url,$origurl,$bugurl,$faqbaseurl,$requestmail,$stayOnPage) = @_;
    my $bodytag = &Apache::loncommon::bodytag('',$function,'rightmargin="0" leftmargin="0" marginwidth="0" topmargin="1" marginheight="1"',1);
    $bodytag=~s/[\n\r]/ /g;
    my $fontcolor = &Apache::loncommon::designparm($function.'.font');
    my $alinkcolor = &Apache::loncommon::designparm($function.'.alink');
    my $vlinkcolor = &Apache::loncommon::designparm($function.'.vlink');
    my $pagecolor = &Apache::loncommon::designparm($function.'.pgbg');
    my $tablecolor = &Apache::loncommon::designparm($function.'.tabbg');
    my $location=&Apache::loncommon::lonhttpdurl("/adm");
    if (($tablecolor eq '') || ($tablecolor eq '#FFFFFF')) {
        $tablecolor = '#EEEE99';
    }
    my $scripttag = '';
    if ($requestmail =~ m/^[^\@]+\@[^\@]+$/) {
        my $displayurl = &Apache::lonnet::escape($origurl);
        $scripttag = (<<"SCRIPT_ONE");
<script>
function gohelpdesk() {
    var actiontype = null;
    try {
        actiontype = parent.bodyframe.document.logproblem.action.value;
    }
    catch(error) {
        parent.bodyframe.location = "/adm/support?origurl=$displayurl&function=$function";
        return;
    }
    if (actiontype) {
        var loc = parent.bodyframe.location.href;
        if (loc.indexOf("/adm/support") > -1) {
            if (parent.bodyframe.document.logproblem.action.value == "process") {
                if (validmail(parent.bodyframe.document.logproblem.email) == false) {
                    alert("The e-mail address you entered: "+parent.bodyframe.document.logproblem.email.value+" is not a valid e-mail address.");
                    return;
                }
                parent.bodyframe.document.logproblem.submit();
                return;
            }
        }
        parent.bodyframe.location = "/adm/support?origurl=$displayurl&function=$function";
        return;
    }
}
SCRIPT_ONE
        $scripttag .= (<<'SCRIPT_TWO');
function validmail(field) {
    var str = field.value;
    if (window.RegExp) {
        var reg1str = "(@.*@)|(\\.\\.)|(@\\.)|(\\.@)|(^\\.)";
        var reg2str = "^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$";
        var reg1 = new RegExp(reg1str);
        var reg2 = new RegExp(reg2str);
        if (!reg1.test(str) && reg2.test(str)) {
            return true;
        }
        return false;
    }
    else
    {

        if(str.indexOf("@") >= 0) {
            return true;
        }
        return false;
    }
}
</script>
SCRIPT_TWO
    }
    my $html=&Apache::lonxml::xmlbegin();
    $r->print(<<END);
$html
<head>
<title>Help Banner</title>
<style type="text/css">
<!--
 a:link {text-decoration: none; color: $fontcolor; }
 a:visited {text-decoration: none; color: $fontcolor; }
 a:active {text-decoration: none; color: $fontcolor; }
 a:hover {text-decoration: underline; color: $vlinkcolor; }
-->
</style>
$scripttag
</head>
$bodytag
END
    if ($stayOnPage) {
        $r->print('<table width="700" border="0" cellspacing="0" cellpadding="0" height="55">');
    } else {
        $r->print('<table width="600" border="0" cellspacing="0" cellpadding="0" height="55">');
    }
    $r->print(<<END);
  <tr height="50">
   <td width='5'>&nbsp;</td>
   <td>
    <fieldset><legend><img src="$location/lonIcons/minilogo.gif" height='20' width='29' valign='bottom' />&nbsp;&nbsp;<b><font size="+1">LON-CAPA help/support</font></b></legend>
 <table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#000000">
  <tr>
   <td>
    <table width="100%" border="0" cellpadding="0" cellspacing="1" bgcolor="#000000">
     <tr>
      <td>
       <table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="#ffffff">
        <tr>
         <td>
	  <table width="100%" border="0" cellpadding="0" cellspacing="1" bgcolor="#ffffff">
           <tr bgcolor="$tablecolor">
END
    if ($component_url) {
	$r->print("<td align=\"center\"><b><a href=\"$component_url\" target=\"bodyframe\">".
		  '<img src="'.$location.'/help/gif/smallHelp.gif" border="0" alt="(Topic help)" valign="middle" />&nbsp;Topic help</a></b>&nbsp;</td>');
    }
    if ($requestmail =~ m/^[^\@]+\@[^\@]+$/) {
        $r->print('
            <td align="center">&nbsp;<b><a href="javascript:gohelpdesk()"><img src="'.$location.'/lonIcons/helpdesk.gif" border="0" alt="(Ask helpdesk)" valign="middle" />&nbsp;Ask helpdesk</a></b>&nbsp;</td>');
    }
    if ($faq && $env{'user.adv'}) {
        $r->print(<<END);
            <td align="center">
             &nbsp;<b><a href="$faqbaseurl/fom/cache/$faq.html" target="bodyframe"><img src="$location/lonMisc/smallFAQ.gif" border="0" alt="(FAQ)" valign="middle" />&nbsp;FAQ</a></b>&nbsp;
            </td>
END
    }
    if ($env{'user.adv'}) {
        $r->print(<<END);
            <td align="center">&nbsp;<b><a href="$bugurl" target="bodyframe"><img src="$location/lonMisc/smallBug.gif" border="0" alt="(Report a bug)" valign="middle" />&nbsp;Report a bug</a>&nbsp;</b></td>
END
    }
    if ($stayOnPage) {
        $r->print(<<END);
            <td align="center">&nbsp;<b><a href="$origurl" target="_top"><img src="$location/lonIcons/move_up.gif" border="0" alt="(Return to last location)" valign="middle" />&nbsp;Return to last location</a></b>&nbsp;</td>
END
    } else {
        $r->print(<<END);
            <td align="center">&nbsp;<b><a href="javascript:window.close()" target="_top"><img src="$location/lonIcons/closepage.gif" border="0" alt="(Close window)" valign="bottom" />&nbsp;Close</a></b>&nbsp;</td>
END
    }
    $r->print(<<END);
           </tr>
          </table>
         </td>
        </tr>
       </table>
      </td>
     </tr>
    </table>
   </td>
  </tr>
 </table>
</fieldset>
  </td>
  <td width='5'>&nbsp;</td>
 </tr>
 <tr height='5'>
  <td colspan='3' height='5'>&nbsp;</td>
 </tr>
</table>
</body>
</html>
END
}

sub display_help_mainpage {
    my ($r,$color,$function,$faq,$bug,$topic,$component_url,$origurl,$bugurl,$faqbaseurl,$requestmail) = @_;
    my $bodytag = &Apache::loncommon::bodytag('',$function,'topmargin="0" marginheight="0"',1);
    $bodytag=~s/[\n\r]/ /g;
    my %lt =&Apache::lonlocal::texthash(
        'heco' => 'Help Content',
        'topp' => 'Topic Page',
        'chen' => 'Choose an entry below to go directly to a relevant help page',
        'orto' => 'or to submit a help request to the LON-CAPA support staff at your institution.',
        'vthp' => 'Visit the help page for ',
        'disp' => 'Display the page in the inline help system that covers this topic.',
        'crac' => 'Create an account for yourself in the LON-CAPA Bugzilla tracking system, if you wish to report bugs you have encountered in the LON-CAPA software, or if you have suggestions for improvements in LON-CAPA.',
        'inhs' => 'Topical help system for',
	'noto' => 'There is no specific topical help item for',
        'coth' => 'Consult the inline help system for this topic.',
        'cont' => 'Contact the LON-CAPA support team',
        'suhr' => 'Submit a help request to the team responsible for LON-CAPA support at this institution.',
        'stuq' => 'Questions about course content should not be directed to the support team, but instead should be sent to the course instructor',
        'faqo' => 'FAQ-O-Matic Help system',
        'tfaq' => 'The FAQ-O-Matic is a compendium of answers provided to common questions asked by users of LON-CAPA over the past couple of years.',
        'lbug' => 'LON-CAPA Bugzilla bug/feature request tracking system',
        'crea' => 'Create an account for yourself in the LON-CAPA Bugzilla tracking system, if you wish to report bugs you have encountered in the LON-CAPA software,or if you have suggestions for improvements in LON-CAPA.',
        'suim' => ' Suggested improvements may include additional functionality, improved usability, or changes to wording used in LON-CAPA pages, including the embedded help system.'
    );
    my $html=&Apache::lonxml::xmlbegin();
    $r->print(<<END);
$html
 <head>
  <title>$lt{'heco'}</title>
 </head>
$bodytag
END
    $r->print($lt{'chen'});
    if ($requestmail =~ m/^[^\@]+\@[^\@]+$/) {
        $r->print(', '.$lt{'orto'});
    } else {
        $r->print(".");
    }
    if ($topic) {
        if ( ($component_url) || ($env{'user.adv'}) ) {
            if ($component_url) {
                $r->print("
          <ul>
           <li><a href=\"$component_url\">$lt{'vthp'} $topic</a></li>
          </ul>
          <p>$lt{'disp'}</p>
                ");
            } elsif ($env{'user.adv'}) {
                $r->print("
              <ul>
               <li><td align=\"center\">&nbsp;<a href=\"/adm/help/nohelptopic.html\">$lt{'noto'} $topic</a></li></ul>
               <p>$lt{'coth'}</p>");
            }
        }
    }
    if ($requestmail =~ m/^[^\@]+\@[^\@]+$/) {
        $r->print("
          <ul>
           <li><a href=\"/adm/support?origurl=".&Apache::lonnet::escape($origurl)."&function=$function\">$lt{'cont'}</a></li>
          </ul>
          <p>$lt{'suhr'}");
         unless ($env{'user.adv'}) {
             $r->print('<br /><b>'.&mt('Note').'</b>: '.$lt{'stuq'}.'. '.&mt('This can be done using the').' <a href="/adm/communicate" target="_top">'.&mt('COM').'</a> '.&mt('button, or the FDBK button when viewing a content page.'));
         }
         $r->print("</p>");
    }
    if ($faqbaseurl && $env{'user.adv'}) {
        if (!defined($faq) ||$faq eq '') {
            $faq = '1';
        }
        $r->print("
          <ul>
           <li><a href=\"$faqbaseurl/fom/cache/$faq.html\">$lt{'faqo'}</a></li>
          </ul>
          <p>$lt{'tfaq'}</p>
        ");
    }
    if ($bugurl && $env{'user.adv'}) {
        $bugurl .= '?'.$bug;
        $r->print("
          <ul>
           <li><a href=\"$bugurl\">$lt{'lbug'}</a></li>
          </ul>
          <p>$lt{'crea'} $lt{'suim'}</p>
        ");
    }
    $r->print(<<END);
 </body>
</html>
END
}

1;
