#!/usr/bin/perl
# CGI-script to run LaTeX, dvips, ps2ps, ps2pdf etc.
#
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

use lib '/home/httpd/lib/perl';
use LONCAPA::loncgi;
use File::Path;
use IO::File;
use Image::Magick;
use Apache::lonhtmlcommon;
use Apache::lonnet;
use Apache::loncommon;
use Apache::lonlocal;
use Apache::lonmsg;
use LONCAPA::Enrollment;

use strict;


#
#   Determine if a user is operating as a student for this course/domain.
#
#
#Parameters:
#   course  - The course id.
#   cdom    - The course domain.
#
#Implicit:
#    $env{request.role} contains the role under which this user operated this
#                       this request.
sub is_student {
    return (! $env{'request.role.adv'});
}

#
#   Debugging:  Dump the environment for debugging.
#
sub dumpenv  {
    print "<br />-------------------<br />";
    foreach my $key (sort (keys %env)) {
	print "<br />$key -> $env{$key}";
    }
    print "<br />-------------------<br />";
}

#
#   This sub sends a message to the appropriate person if there was an error
#   rendering the latex  At present, there's only one case to consider:
#   a student printing inside a course results in messages to the course coordinator.
#Parmaeters:
#    identifier -  The unique identifier of this cgi request.
#    badresource-  Filepath to most likely failing 
#    logfile    -  The contents of the log file (included in the message).
#    texfile    -  reference to an array containing the LaTeX input file
#                  (included in the message).
#Implicit inputs:
#   From the environment:
#       cgi.$identifier.user     - User doing the printing.
#       cgi.$identifier.domain   - Domain the user is logged in on with printing.
#       cgi.$identifier.courseid - Id of the course (if this is a course).
#       cgi.$identifier.coursedom- Domain in which course was constituted.
#       cgi.$identifier.resources - List of resource URL's for which the print
#                                  was attempted.
# 
sub send_error_mail {
    my ($identifier, $badresource, $logfile, $texfile) = @_;
    my $user     = $env{"cgi.$identifier.user"};
    my $domain   = $env{"cgi.$identifier.domain"};
    my $courseid = $env{"cgi.$identifier.courseid"};
    my $coursedom= $env{"cgi.$identifier.coursedom"};
    my $resources= $env{"cgi.$identifier.resources"};

    #  resource file->URL
    #
    my $badurl = &Apache::lonnet::declutter($badresource);

    # &dumpenv();



    #  Only continue if there is a user, domain, courseid, course domain
    #  and resources:

    if(defined($user)       && defined($domain) && defined($courseid) &&
       defined($coursedom)  && defined($resources) ){
	   
	#  Only mail if the conditions are ripe for it:
	#  The user is a student in the course:
	#
	
	if (&is_student( $courseid, $coursedom)) {
	    # build the subject and message body:
	    # print "sending message to course coordinators.<br />";

	    # Todo: Convert badurl into a url from file path:

	    my $subject  = "Error [$badurl] Print failed for $user".'@'.$domain;
	    my $message .= "Print failed to render LaTeX for $user".'@'."$domain\n";
	    $message    .= "  User was attempting to print: \n";
	    $message    .= "       $resources\n";
	    $message    .= "--------------------LaTeX logfile:------------ \n";
	    $message    .= $logfile;
	    $message    .= "-----------------LaTeX source file: ------------\n";
	    
	    foreach my $line (@$texfile) {
		$message .= "$line\n";
	    }
	    my (undef, %receivers) = &Apache::lonfeedback::decide_receiver(undef, 0,
									  1,1,1);
	    # print "<br /> sending...section:  $env{'request.course.sec'}";
	    foreach my $dest (keys %receivers) {
		# print "<br /> dest is $dest";
		my @destinfo = split(/:/,$dest);
		my $user = $destinfo[0];
		my $dom  = $destinfo[1];

		&Apache::lonmsg::user_normal_msg($user, $dom,
						 $subject, $message);
		
		# No point in looking at the return status as there's no good
		# error action I can think of right now (log maybe?).
	    }
	}
    }
}

$|=1;
if (! &LONCAPA::loncgi::check_cookie_and_load_env()) {
    print <<END;
Content-type: text/html

<html>
<head><title>Bad Cookie</title></head>
<body>
Your cookie information is incorrect.
</body>
</html>
END
    return;
}
 &Apache::lonlocal::get_language_handle();
 &Apache::loncommon::content_type(undef,'text/html');
 my $bodytag=&Apache::loncommon::bodytag('Creating PDF','','');
 print $bodytag;

  my $identifier = $ENV{'QUERY_STRING'};
  my $texfile = $env{'cgi.'.$identifier.'.file'};
  my $laystyle = $env{'cgi.'.$identifier.'.layout'};
  my $numberofcolumns = $env{'cgi.'.$identifier.'.numcol'};
  my $paper = $env{'cgi.'.$identifier.'.paper'};
  my $selectionmade = $env{'cgi.'.$identifier.'.selection'};
  my $tableofcontents = $env{'cgi.'.$identifier.'.tableofcontents'};
  my $tableofindex = $env{'cgi.'.$identifier.'.tableofindex'};
  my $advanced_role = $env{'cgi.'.$identifier.'.role'};
  my $number_of_files = $env{'cgi.'.$identifier.'.numberoffiles'}+1;
  my $student_names = $env{'cgi.'.$identifier.'.studentnames'};
  my $backref = &Apache::lonnet::unescape($env{'cgi.'.$identifier.'.backref'});


  my $adv = $env{'request.role.adv'};
  
  my @names_pack=();
  if ($student_names=~/_END_/) {  
      @names_pack=split(/_ENDPERSON_/,$student_names);
  }

  my $figfile = $texfile;
  $figfile =~ s/^([^\.]+printout)[^t]+\.tex/$1\.dat/;
  my $duefile = $texfile;
  $duefile =~ s/^([^\.]+printout)[^t]+\.tex/$1\.due/;
  #do we have figures?
  # print "Figure file: $figfile\n";
  if (-e $figfile) {
      # print "$figfile exists\n";
      my %done_conversion;
      my $temporary_file=IO::File->new($figfile) || die "Couldn't open fig file $figfile for reading: $!\n";
      my @content_of_file = <$temporary_file>;
      close $temporary_file;  
      my $noteps;
      my %prog_state;
      if ($adv) { %prog_state=&Apache::lonhtmlcommon::Create_PrgWin('','Coverting Images to EPS','Picture Conversion Status',$#content_of_file,'inline','80');  }
      foreach my $not_eps (@content_of_file) {
	  chomp($not_eps);
	  if ($not_eps ne '') {
	      # print "Converting $not_eps"; # Debugging.
              my $status_statement='EPS picture for '.$not_eps;
	      # print "$status_statement\n";
	      $not_eps=~s|\/\.\/|\/|g;
	      my $eps_f = $not_eps;
	      $eps_f =~ s/\.[^.]*$/\.eps/i;
	      if ($eps_f=~/\/home\/([^\/]+)\/public_html\//) {
                  $eps_f=~s/\/home\/([^\/]+)\/public_html/$1/;
		  $eps_f = '/home/httpd/prtspool/'.$eps_f;
	      } elsif ($eps_f=~/$Apache::lonnet::perlvar{'lonDocRoot'}\/res\//) {
		  $eps_f=~m/$Apache::lonnet::perlvar{'lonDocRoot'}\/res\/(.+)/;
		  $eps_f = '/home/httpd/prtspool/'.$1;
	      } elsif ($eps_f=~/$Apache::lonnet::perlvar{'lonUsersDir'}\//) {
		  $eps_f=~/$Apache::lonnet::perlvar{'lonUsersDir'}\/([^\/]+)\/\w\/\w\/\w\/(.+)/;
		  $eps_f = '/home/httpd/prtspool/'.$1.'/'.$2;
	      }
	      $eps_f  =~ s/ /\_/g; # Spaces are problematic for system commands and LaTeX.
	      my $path=$eps_f;
	      $path =~ s/\/([^\/]+)\.eps$//;
	      # print "Final file path: $path "; # Debugging
	      File::Path::mkpath($path,0,0777);
	      $not_eps =~ s/^\s+//;
	      $not_eps =~ s/\s+$//;
	      $not_eps =~ s/ /\\ /g;
	      if ( exists($done_conversion{$not_eps})) { next; }
	      if ($adv) {
		  my $prettyname=$not_eps;
		  $prettyname=~s|/home/([^/]+)/public_html|/priv/$1|;
		  $prettyname=~s|$Apache::lonnet::perlvar{'lonDocRoot'}/|/|;
		  &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,'Converting to EPS '.$prettyname);
	      }
	      $done_conversion{$not_eps}=1;
	      # print "Converting $not_eps -> $eps_f"; # Debugging
	      system("convert $not_eps $eps_f");
              #check is eps exist in prtspool
              if(not -e $eps_f) {
		  for (my $i=0;$i<10000;$i++) {
		      if (-e $eps_f.'.'.$i) {
			  rename $eps_f.'.'.$i, $eps_f;
			  last;
		      }
		  }
	      }  
	  }
      }
      if ($adv) { 
	  &Apache::lonhtmlcommon::Close_PrgWin('',\%prog_state); 
      }
      unlink($figfile);
  }
  #print "$texfile\n"; #name of the tex file for debugging only   
  my @texfile=($texfile);
  if ($number_of_files>1) {
      @texfile=();
      for (my $i=1;$i<=$number_of_files;$i++) {
	  my $new_texfile=$texfile;
	  $new_texfile=~s/\.tex/_$i\.tex/;
	  push @texfile,$new_texfile;
      } 
  }

my $ind=-1;
my %prog_state;
print "<a href=\"$backref\"><b>Return</b></a> to last resource.<br /><br />";
if ($adv) { %prog_state=&Apache::lonhtmlcommon::Create_PrgWin('','Print Status','Class Print Status',$number_of_files,'inline','80'); }
print "<br />";
foreach $texfile (@texfile) {
  my $status_statement='';
  my $link_text='download PDF';
  $ind++;
  my @stud_info=split(/_END_/,$names_pack[$ind]);
  my @tempo_array=split(/:/,$stud_info[0]);
  my $name;
  if ($tempo_array[3]) {
      $name=$tempo_array[3];
  } else {
      $name=$tempo_array[0].'@'.$tempo_array[1];
  }
  $link_text='<b>'.$name.'</b>';
  $status_statement.=$name;
  if ($#stud_info>0) {
      @tempo_array=split(/:/,$stud_info[-1]);
      if ($tempo_array[3]) {
	  $name=$tempo_array[3];
      } else {
	  $name=$tempo_array[0].'@'.$tempo_array[1];
      }
      $link_text.=' - <b>'.$name.'</b>';
      $status_statement.=' -  '.$name;
  } 
  if ($adv) { &Apache::lonhtmlcommon::Update_PrgWin('',\%prog_state,'Creating PDF for '.$status_statement); }
  if (-e $texfile) {
      $texfile =~ m/^(.*)\/([^\/]+)$/; 
      my $name_file = $2;
      my $path_file = $1.'/';
      chdir $path_file;
      my $dvi_file= $name_file; $dvi_file =~ s/\.tex/\.dvi/;
      &busy_wait_command("latex $name_file 1>/dev/null 2>/dev/null",
			 "for $status_statement now LaTeXing file",
			 \%prog_state,$dvi_file);
      if ($tableofcontents eq 'yes') {
      &busy_wait_command("latex $name_file 1>/dev/null 2>/dev/null",
			 "for $status_statement now LaTeXing file for table of contents",
			 \%prog_state,$dvi_file);
      } #to create table of contents
      my $idxname=$name_file;
      $idxname=~s/\.tex$/\.idx/;
      if ($tableofindex eq 'yes') {
	  &busy_wait_command("makeindex $idxname",
			     "making index file",
			     \%prog_state,$idxname);
	  &busy_wait_command("latex $name_file 1>/dev/null 2>/dev/null",
			     "for $status_statement now LaTeXing file for index section",
			     \%prog_state,$dvi_file);
      } #to create index
      #Do we have a latex error in the log file?
      my $logfilename = $texfile; $logfilename =~ s/\.tex$/\.log/;
      my $temporary_file=IO::File->new($logfilename) || die "Couldn't open log file $logfilename for reading: $!\n";
      my @content_of_file = <$temporary_file>;
      close $temporary_file; 
      my $body_log_file = join(' ',@content_of_file);
      $logfilename =~ s/\.log$/\.html/;
      $temporary_file = IO::File->new('>'.$logfilename); 
      print $temporary_file '<html><head><title>LOGFILE</title></head><body><pre>'.$body_log_file.'</pre></body></html>'."\n";
      if ($body_log_file=~m/!\s+Emergency stop/) {
	  my $whereitbegins = rindex $body_log_file,'STAMPOFPASSEDRESOURCESTART';
	  my $whereitends = rindex $body_log_file,'STAMPOFPASSEDRESOURCEEND';
	  my $badresource;
	  my $badtext;
	  if ($whereitbegins!=-1 and $whereitends!=-1) {
	      $badtext = substr($body_log_file,$whereitbegins+26, $whereitends-$whereitbegins-26);
	      $whereitbegins  = rindex $badtext,'located in';
	      if ($whereitbegins != -1) {
		  
		  $badresource = substr($badtext, $whereitbegins+27, 
					length($badtext) - $whereitbegins - 48);
		  # print "<br />failing resourcename: $badresource<br />";
	      }
	  }
	  
          if ($advanced_role) {  
	      #LaTeX failed to parse tex file 
	      print "<h2>LaTeX could not successfully parse your tex file.</h2>";
	      print "It probably has errors in it.<br />";
	      print "With very high probability this error occured in ".$badtext."<br /><br />";
	      print "Here are the error messages in the LaTeX log file<br /><pre>";
	      my $sygnal = 0;
	      for (my $i=0;$i<=$#content_of_file;$i++) {
		  if ($content_of_file[$i]=~m/^Runaway argument?/ or $content_of_file[$i]=~m/^!/) {
		      $sygnal = 1;
		  } 
		  if ($content_of_file[$i]=~m/Here is how much of/) {
		      $sygnal = 0;
		  } 
		  if ($sygnal) {
		      print "$content_of_file[$i]";
		  }  
	      }
	      print "</pre>\n";
	      # print "<br /> Advanced role <br />";
              print "<b><big>The link to ";
              $logfilename=~s/\/home\/httpd//;
	      print "<a href=\"$logfilename\">Your log file </a></big></b>";
	      print "\n";
              #link tooriginal LaTeX file (included according Michael Hamlin desire)
	      my $tex_temporary_file=IO::File->new($texfile) || die "Couldn't open tex file $texfile for reading: $!\n";
	      my @tex_content_of_file = <$tex_temporary_file>;
	      close $tex_temporary_file; 
	      my $body_tex_file = join(' ',@tex_content_of_file);
	      $texfile =~ s/\.tex$/aaaaa\.html/;
	      $tex_temporary_file = IO::File->new('>'.$texfile); 
	      print $tex_temporary_file '<html><head><title>LOGFILE</title></head><body><pre>'.$body_tex_file.'</pre></body></html>'."\n";
	      print "<br /><br />";
	      print "<b><big>The link to ";
	      $texfile=~s/\/home\/httpd//;
	      print "<a href=\"$texfile\">Your original LaTeX file </a></big></b>";
	      print "\n";
	  } else {		# Student role...
	      #  at this point:
	      #    $body_log_file - contains the log file.
              #    $name_file     - is the name of the LaTeX file.
              #    $identifier    - is the unique LaTeX identifier.l

	      print "<br />There are errors in $badtext";
	      print "<br />These errors prevent this resource from printing correctly";
	      my $tex_handle = IO::File->new($name_file);
	      my @tex_contents = <$tex_handle>;
	      &send_error_mail($identifier, $badresource, $body_log_file, \@tex_contents);
	      print "<br />A message has been sent to the instructor describing this failure<br />"
	  }

      } elsif ($body_log_file=~m/<inserted text>/) {
	  my $whereitbegins = index $body_log_file,'<inserted text>';
	  print "You are running LaTeX in <b>batch mode</b>.";
	  while ($whereitbegins != -1) {
	      my $tempobegin=$whereitbegins;
	      $whereitbegins = rindex $body_log_file,'STAMPOFPASSEDRESOURCESTART',$whereitbegins;
	      my $whereitends = index $body_log_file,'STAMPOFPASSEDRESOURCEEND',$whereitbegins;
	      print "<br />It has found an error in".substr($body_log_file,$whereitbegins+26,$whereitends-$whereitbegins-26)." <br /> and corrected it.\n";
	      print "Usually this correction is valid but you probably need to check the indicated resource one more time and implement neccessary corrections by yourself.\n";
	      $whereitbegins = index $body_log_file,'<inserted text>',$tempobegin+10;
	  }
	  $name_file =~ s/\.tex/\.dvi/;
	  my $new_name_file = $name_file;
	  $new_name_file =~ s/\.dvi/\.ps/;
	  my $papera=$paper;
	  if ($papera eq 'letter') {$papera='';}
	  if ($papera ne '') {$papera='-t'.$papera;}
	  my $comma = "dvips $papera -Ppdf -G0 -o $new_name_file";
	  &busy_wait_command("$comma $name_file 1>/dev/null 2>/dev/null",
			     "for $status_statement now Converting to PS",
			     \%prog_state,$new_name_file);
	  if (-e $new_name_file) {
	      print "<h1>PDF output file (see link below)</h1>\n";
	      $new_name_file =~ m/^(.*)\./;
	      my $ps_file = my $tempo_file = $1.'temporar.ps';
	      my $pdf_file = $1.'.pdf';
	      if ($laystyle eq 'album' and $numberofcolumns eq '2') {
		  my $papera=$paper;
                  if ($papera eq 'letter') {$papera='';}
		  if ($papera ne '') {$papera='-p'.$papera;}
		  $comma = "psnup $papera -2 -s1.0 $new_name_file";
		  &busy_wait_command("$comma $tempo_file 1>/dev/null 2>/dev/null",
				     "for $status_statement now Modifying PS layout",
				     \%prog_state,$tempo_file); 
	      } elsif ($laystyle eq 'book' and $numberofcolumns eq '2') {
		  my $papera=$paper;
                  if ($papera eq 'letter') {$papera='';}
		  if ($papera ne '') {$papera='-p'.$papera;}
		  $comma = 'pstops '.$papera.' "2:0+1(0.48w,0)"';
		  &busy_wait_command("$comma $new_name_file $tempo_file 1>/dev/null 2>/dev/null",
				     "for $status_statement now Modifying PS layout",
				     \%prog_state,$tempo_file); 

	      } else {
		  $ps_file=$new_name_file;
	      }	    
	      &busy_wait_command("ps2pdf $ps_file $pdf_file 1>/dev/null 2>/dev/null",
				 "for $status_statement now Converting PS to PDF",
				 \%prog_state, $pdf_file);

	      my $texlog = $texfile;
	      my $texaux = $texfile;
	      my $texdvi = $texfile;
	      my $texps = $texfile;
	      $texlog =~ s/\.tex/\.log/;
	      $texaux =~ s/\.tex/\.aux/;
	      $texdvi =~ s/\.tex/\.dvi/;
	      $texps =~ s/\.tex/\.ps/;
	      my @garb = ($texaux,$texdvi,$texps);
#	  unlink @garb;
	      unlink $duefile;
	      print "<a href=\"/prtspool/$pdf_file\">Your PDF document</a>";
	  }
	  if ($advanced_role) {  
	      print "<br /><br />";
	      print "<b><big>The link to ";
	      $logfilename=~s/\/home\/httpd//;
	      print "<a href=\"$logfilename\">Your log file </a></big></b>";
	      print "\n";
	      #link tooriginal LaTeX file (included according Michael Hamlin desire)
	      my $tex_temporary_file=IO::File->new($texfile) || die "Couldn't open tex file $texfile for reading: $!\n";
	      my @tex_content_of_file = <$tex_temporary_file>;
	      close $tex_temporary_file; 
	      my $body_tex_file = join(' ',@tex_content_of_file);
	      $texfile =~ s/\.tex$/aaaaa\.html/;
	      $tex_temporary_file = IO::File->new('>'.$texfile); 
	      print $tex_temporary_file '<html><head><title>LOGFILE</title></head><body><pre>'.$body_tex_file.'</pre></body></html>'."\n";
	      print "<br /><br />";
	      print "<b><big>The link to ";
	      $texfile=~s/\/home\/httpd//;
	      print "<a href=\"$texfile\">Your original LaTeX file </a></big></b>";
	      print "\n";
	  }
      } else {
	  #LaTeX successfully parsed tex file 
	  $name_file =~ s/\.tex/\.dvi/;
	  my $new_name_file = $name_file;
	  $new_name_file =~ s/\.dvi/\.ps/;
	  my $papera=$paper;
	  if ($papera eq 'letter') {$papera='';}
	  if ($papera ne '') {$papera='-t'.$papera;}
	  my $comma = "dvips $papera -Ppdf -G0 -o $new_name_file";
	  &busy_wait_command("$comma $name_file 1>/dev/null 2>/dev/null",
			     "for $status_statement now Converting to PS",
			     \%prog_state,$new_name_file);
	  if (-e $new_name_file) {
	      print "<br />";
	      $new_name_file =~ m/^(.*)\./;
	      my $ps_file = my $tempo_file = $1.'temporar.ps';
	      my $pdf_file = $1.'.pdf';
	      $papera=~s/t/p/;
	      if ($laystyle eq 'album' and $numberofcolumns eq '2') {
		  $comma = "psnup $papera -2 -s1.0 $new_name_file";
		  &busy_wait_command("$comma $tempo_file 1>/dev/null 2>/dev/null",
				     "for $status_statement now Modifying PS layout",
				     \%prog_state,$tempo_file);
	      } elsif ($laystyle eq 'book' and $numberofcolumns eq '2') {
		  $comma = 'pstops '.$papera.' "2:0+1(0.48w,0)"';
		  &busy_wait_command("$comma $new_name_file $tempo_file 1>/dev/null 2>/dev/null",
				     "for $status_statement now Modifying PS layout",
				     \%prog_state,$tempo_file); 
	      } else {
		  $ps_file=$new_name_file;
	      }
	      my $addtoPSfile={'legal'=>'<< /PageSize [612 1008] >> setpagedevice',
                               'tabloid'=>'<< /PageSize [792 1224] >> setpagedevice',
                               'executive'=>,'<< /PageSize [540 720] >> setpagedevice',
                               'a2'=>'<< /PageSize [1195.02 1690.09] >> setpagedevice',
                               'a3'=>'<< /PageSize [842 1195.02] >> setpagedevice',
                               'a4'=>'<< /PageSize [595.2 842] >> setpagedevice',
                               'a5'=>'<< /PageSize [421.1 595.2] >> setpagedevice',
                               'a6'=>'<< /PageSize [298.75 421.1] >> setpagedevice',
			   };
	      if ($paper ne 'letter') {
		  open(FFH,'<',$ps_file) || die "Couldn't open ps file $ps_file for reading: $!\n";
		  my $new_ps_file='new'.$ps_file;
		  open(FFHS,'>',$new_ps_file) || die "Couldn't open new ps file $new_ps_file for reading: $!\n";
		  print FFHS $addtoPSfile->{$paper}."\n";
		  while (<FFH>) {
		      print FFHS $_;
		  }
		  close(FFH);
		  close(FFHS);
		  $ps_file=$new_ps_file;	  
	      }
	      &busy_wait_command("ps2pdf $ps_file $pdf_file 1>/dev/null 2>/dev/null",
				 "for $status_statement now Converting PS to PDF",
				 \%prog_state,$pdf_file);
	    
	      my $texlog = $texfile;
	      my $texaux = $texfile;
	      my $texdvi = $texfile;
	      my $texps = $texfile;
	      $texlog =~ s/\.tex/\.log/;
	      $texaux =~ s/\.tex/\.aux/;
	      $texdvi =~ s/\.tex/\.dvi/;
	      $texps =~ s/\.tex/\.ps/;
	      my @garb = ($texlog,$texaux,$texdvi,$texps);
#	  unlink @garb;
	      unlink $duefile;
	      print "<a href=\"/prtspool/$pdf_file\">$link_text - click here to download pdf</a>";
	      print "\n";
	  }
      }  
  } else {
      print "LaTeX file $texfile was not created successfully";
  }
}
print "<br />";
if ($number_of_files>1) {
    my $zipfile=$texfile[0];
    $zipfile=~s/\.tex/\.zip/;
    my $statement="zip $zipfile";
    foreach my $file (@texfile) {
	$file=~s/\.tex/.\pdf/;
	$statement.=' '.$file; 
    }
    print("<pre>Zip Output:\n");
    system($statement);
    print("</pre>");
    $zipfile=~s/\/home\/httpd//;
    print "<br /> A <a href=\"$zipfile\">ZIP file</a> of all the PDFs.";
}
if ($adv) { &Apache::lonhtmlcommon::Close_PrgWin('',\%prog_state); }

my $done;
sub REAPER {
    $done=1;
}

sub busy_wait_command {
    my ($command,$message,$progress_win,$output_file)=@_;
    
    $SIG{CHLD} = \&REAPER;
    $done=0;
    my $pid=open(CMD,"$command |");
    if ($adv) {
	&Apache::lonhtmlcommon::Update_PrgWin('',$progress_win,$message);
    }
    while(!$done) {
	sleep 1;
	my $extra_msg;
	if ($output_file) {
	    my $size=(stat($output_file))[7];
	    $extra_msg=", $size bytes generated";
	}
	if ($adv) {
	    &Apache::lonhtmlcommon::Update_PrgWin('',$progress_win,
						  $message.$extra_msg);
	}
    }
    $SIG{CHLD}='IGNORE';
    close(CMD);
}




